;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PICmicro PIC16F877. This file contains the basic code      *
;   building blocks to build upon.                                    *  
;                                                                     *
;   If interrupts are not used all code presented between the ORG     *
;   0x004 directive and the label main can be removed. In addition    *
;   the variable assignments for 'w_temp' and 'status_temp' can       *
;   be removed.                                                       *                         
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PICmicro data sheet for additional        *
;   information on the instruction set.                               *
;                                                                     *
;   Template file assembled with MPLAB V4.00 and MPASM V2.20.00.      *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	   timer.asm                                          *
;    Date:         14/06/2544                                         *
;    File Version: 1.00                                               *
;                                                                     *
;    Author:       PITAK TISUTHIWONGSE                                *
;    Company:                                                         *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;     OSC = 4 MHZ                                                     *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************


	list      p=16f877            ; list directive to define processor
	#include <p16f877.inc>        ; processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_ON & _BODEN_ON & _PWRTE_ON & _RC_OSC & _WRT_ENABLE_ON & _LVP_ON & _DEBUG_OFF & _CPD_OFF 

; '__CONFIG' directive is used to embed configuration data within .asm file.
; The lables following the directive are located in the respective .inc file.
; See respective data sheet for additional information on configuration word.

;------------------------------------------------------------------------------
; LCD PORT DEFINITION
;------------------------------------------------------------------------------
LCD_Type		EQU	(d'2'*d'16')	; 2x16 = 32 Char
LCD_DATA		EQU	PORTD		; DATA bit4 - bit7 of LCD (PD.0 - PD.3)
#define _LCD_BUSY_		PORTD,7		; LCD busy flag
#define _LCD_RS_		PORTA,0		; LCD Register Select (PA.0)
#define _LCD_RW_		PORTA,1		; LCD Read/Write Select (PA.1)
#define	_LCD_EN_		PORTA,2		; LCD Enable port (PA.2)
;-----------------------------------------------------------------------------
; VARIABLE DEFINITIONS
;-----------------------------------------------------------------------------
CountTime1		EQU     0x22		; use for delay count time1
CountTime2		EQU     0x23	  	; use for delay count time2
Temp1			EQU	0x24		; Temp1 data
Temp2			EQU	0x25		; Temp2 data
Temp3			EQU	0x26		; Temp3 data
TempBit			EQU	0x27		; Temp bit bit 0-7
;
w_temp			EQU     0x70        	; variable used for context saving 
status_temp		EQU     0x71        	; variable used for context saving
;**********************************************************************
		ORG     0x000             ; processor reset vector
		nop
		clrf    PCLATH            ; ensure page bits are cleared
  		goto    main              ; go to beginning of program


		ORG     0x004             ; interrupt vector location
		movwf   w_temp            ; save off current W register contents
		movf	STATUS,w          ; move status register into W register
		movwf	status_temp       ; save off contents of STATUS register
		;
		bcf	PIR1,TMR1IF
		call	Timer1_Intr	  ; Call Timer1 Interrupt
		;
		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt

;-----------------------------------------------------------------------------
;			 MAIN ROUTINE START HERE
;-----------------------------------------------------------------------------
	        ORG     0x0020
main:
		call	Delay_5mSec		; Warm up
;-----------------------------------------------------------------------------
; NAME		: Init_SYSTEM
; PURPOSE	: Initialize program
;-----------------------------------------------------------------------------
Init_program:
	call    Init_Timer		; Initialize timer register
        call    Init_Ports		; Initialize PORT of M05
	movlw	b'11000000'		
	movwf	INTCON
	bsf     STATUS,5		; Select bank 1
	bcf	STATUS,6
	movlw	b'00000001'		; Enable Timer1 Interrupt
	movwf	PIE1
	bcf     STATUS,5		; Select bank 0
	bcf	STATUS,6
	clrf	PORTA
	movlw	h'FF'			; Drive LOW set all PORTD HIGH
	movwf	PORTD
	call	InitLCD_2x16		; Initialize LCD 2x16
	call	Write_LCDMsg
;-----------------------------------------------------------------------------
; Program loop start here
;-----------------------------------------------------------------------------
StartLoop:
	goto	StartLoop
;-----------------------------------------------------------------------------
; Timer1 Interrupt
;-----------------------------------------------------------------------------
Timer1_Intr:
	;
	return
;-----------------------------------------------------------------------------
; Write LCD message
;-----------------------------------------------------------------------------
Write_LCDMsg:
		bcf    _LCD_RS_		; Set Write command
		call	WaitNot_Busy	; LCD set Initialize
		movlw	b'10000000'	; Start address at 0
		movwf	LCD_DATA
		call	EN_1Pulse
		bsf    _LCD_RS_		; Set Write data
		clrf	Temp1		; Clear count
Write_LCDMsg1:				; Write First line
		movfw	Temp1
		call    Load_Msg
		movwf	Temp3
		call	WaitNot_Busy	; Wait not busy
		movfw	Temp3		; Send ASCII to LCD	
		movwf	LCD_DATA
		call	EN_1Pulse
		incf	Temp1,f
		movlw	d'16'
		xorwf	Temp1,w
		bnz	Write_LCDMsg1
		;
		bcf     _LCD_RS_	; Set Write command
		bcf	_LCD_RW_	; Set write mode	
		call	WaitNot_Busy	; Wait not busy
		movlw	b'11000000'	; Set for Line2
		movwf	LCD_DATA
		call	EN_1Pulse
		bsf     _LCD_RS_	; Set Write data
		movlw	d'16'
		movwf	Temp1
Write_LCDMsg2:				; Write Second line
		movfw	Temp1
		call    Load_Msg
		movwf	Temp3
		call	WaitNot_Busy	; Wait not busy
		movfw	Temp3		; Send ASCII to LCD	
		movwf	LCD_DATA
		call	EN_1Pulse
		incf	Temp1,f
		movlw	d'32'
		xorwf	Temp1,w
		bnz	Write_LCDMsg2
		return
;-----------------------------------------------------------------------------
; Load LCD message from program memory
;-----------------------------------------------------------------------------
Load_Msg:
		addwf     PCL,f         ; Table of ASCII and custom characters
					; to display.

		dt	"Hitachi,HD-44780"
		dt	"Test by PIC-ICD "
;-----------------------------------------------------------------------------
; Initialize LCD
;-----------------------------------------------------------------------------
InitLCD_2x16:
	bcf	_LCD_EN_
	bcf	_LCD_RW_
	bcf	_LCD_RS_		; RS low write command to RS
	call	Delay_125uSec		; Delay 125 uSec
	;
	call	WaitNot_Busy		; LCD set Initialize
	movlw	b'00111000'		; Set 8-bit data,2 lines,5x7 font
	movwf	LCD_DATA
	call	EN_1Pulse
	;
	call	WaitNot_Busy		; LCD set display mode
	movlw	b'00001100'		; Display ON,Cursor Off,Blink Off
	movwf	LCD_DATA
	call	EN_1Pulse
	;
	call	WaitNot_Busy		; LCD set entry mode
	movlw	b'00000110'		; Increment Pointer, Cursor Shift
	movwf	LCD_DATA
	call	EN_1Pulse
	;
	call	WaitNot_Busy		; LCD set entry mode
	movlw	b'00000011'		; set LCD return to HOME
	movwf	LCD_DATA
	call	Delay_5mSec
	return
;-----------------------------------------------------------------------------
; Generate 1 pulse at enable pin
;-----------------------------------------------------------------------------
EN_1Pulse:
	bsf	_LCD_EN_
	nop
	bcf	_LCD_EN_
	return
;-----------------------------------------------------------------------------
; Wait until not busy
;-----------------------------------------------------------------------------
WaitNot_Busy:
		bcf	TempBit,0	; Store RS status
		btfsc	_LCD_RS_
		bsf	TempBit,0
		;
		bcf	_LCD_RS_	; Set to send command mode
		bsf     _LCD_RW_        ; Set to read mode 
		bsf     STATUS,5	; Select bank 1
		bcf	STATUS,6
		movlw   b'11111111'	; Set LCD DATA port to input status
		movwf   TRISD
		bcf     STATUS,5	; Select bank 0
		bcf	STATUS,6
		;
WaitNot_Busy1:
		bsf	_LCD_EN_	; Enable LCD	
		nop
		movfw	LCD_DATA	; Load LCD data to Temp Register 
		movwf	Temp2
		bcf     _LCD_EN_        ; Disable LCD. 
		btfsc	Temp2,7
		goto	WaitNot_Busy1	; Loop until LCD not busy
		bcf	_LCD_RS_	; Restore RS status
		btfsc	TempBit,0
		bsf	_LCD_RS_
		bcf     _LCD_RW_        ; Set to write mode 
		bsf     STATUS,5	; Select bank 1
		bcf	STATUS,6
		movlw   b'00000000'	; Set LCD DATA port to output status
		movwf   TRISD
		bcf     STATUS,5	; Select bank 0
		bcf	STATUS,6
		return
;-----------------------------------------------------------------------------
; Delay 150 usec (for OSC 4MHz)
;-----------------------------------------------------------------------------
Delay_125uSec:
	movlw	40			; 1 cy = 1 uSec
	movwf	CountTime1		; 1 cy = 1 uSec
	decfsz	CountTime1,f		; 1 cy 40*3 = 120 uSec
	goto	$-1			; 2 cy
	return				; 1+2 cy = 3 uSec
;-----------------------------------------------------------------------------
; Delay 5mSec (Have a little error)
;-----------------------------------------------------------------------------
Delay_5mSec:
	movlw   40
	movwf	CountTime2
Delay_5mSec_1:
	call	Delay_125uSec	
	decfsz	CountTime2,f		; 125uSec*40
	goto	Delay_5mSec_1
	return	
;-----------------------------------------------------------------------------
; Initialize Timer
;-----------------------------------------------------------------------------
Init_Timer:
	movlw	b'00111101'			; Fosc/4,Pre=1:8
	movwf	T1CON
        return
;-----------------------------------------------------------------------------
; Initialize Port
;-----------------------------------------------------------------------------
Init_Ports:
	bsf     STATUS,5			; Select bank 1
	bcf	STATUS,6
	bcf     OPTION_REG,5			; Internal intstruction cycle clock (CLKOUT)
	bcf	OPTION_REG,7			; Enable PORTB pull up
	movlw   b'10000111'			; RA0-RA5,RE0-RE2 as DIGITAL I/O
	movwf   ADCON1
;-----------------------------------------------------------------------------
; Port A setup
;-----------------------------------------------------------------------------
;RA0 (OUT1)		act as output
;RA1 (OUT2)		act as output
;RA2 (OUT3)		act as output 
;RA3 (OUT4)		act as output
;RA4 (OUT5) 		act as output
;RA5 (OUT6)		act as output
;-----------------------------------------------------------------------------
	movlw   0x0		
        movwf   TRISA           
;-----------------------------------------------------------------------------
; Port E setup
;-----------------------------------------------------------------------------
;RE0 (not use)		act as output
;RE1 (not use)		act as output
;RE2 (not use)		act as output
;-----------------------------------------------------------------------------
	movlw   b'00000000'
	movwf   TRISE
;-----------------------------------------------------------------------------
; Port B setup
;-----------------------------------------------------------------------------
;RB0 (not use)		act as output
;RB1 (not use)		act as output
;RB2 (not use)		act as output
;RB3 (not use)		act as output
;RB4 (not use)		act as output
;RB5 (not use)		act as output
;RB6 (not use)		act as output
;RB7 (not use)		act as output
;-----------------------------------------------------------------------------
	movlw	b'00000000'
	movwf	TRISB
;-----------------------------------------------------------------------------
; Port C setup
;-----------------------------------------------------------------------------
;RC0 (not use)		act as output
;RC1 (not use)		act as output
;RC2 (not use)		act as output
;RC3 (not use)		act as output
;RC4 (not use)		act as output
;RC5 (not use)		act as output
;RC6 (not use)		act as output
;RC7 (not use)		act as output
;-----------------------------------------------------------------------------
	movlw   b'00000000'
	movwf   TRISC
;-----------------------------------------------------------------------------
; Port D setup
;-----------------------------------------------------------------------------
;RD0 (not use)		act as output
;RD1 (not use)		act as output
;RD2 (not use)		act as output
;RD3 (not use)		act as output
;RD4 (not use)		act as output
;RD5 (not use)		act as output
;RD6 (not use)		act as output
;RD7 (not use)		act as output
;-----------------------------------------------------------------------------
	movlw   b'00000000'
	movwf   TRISD
;-----------------------------------------------------------------------------
	bcf     STATUS,5			; Select page 0
	bcf	STATUS,6
        return

	END                       ; directive 'end of program'

